@@ -39,7 +39,11 @@ my $_build__content = sub {
    my $cl = $self->content_length  or return NUL;
    my $fh = $env->{ 'psgi.input' } or return NUL;
 
-   try   { $fh->seek( 0, 0 ); $fh->read( $content, $cl, 0 ); $fh->seek( 0, 0 ) }
+   try {
+      $fh->can( 'seek' ) and $fh->seek( 0, 0 );
+      $fh->read( $content, $cl, 0 );
+      $fh->can( 'seek' ) and $fh->seek( 0, 0 );
+   }
    catch { $log->( { level => 'error', message => $_ } ); $content = NUL };
 
    return $content || NUL;
@@ -83,7 +87,10 @@ has 'path'           => is => 'lazy', isa => SimpleStr, builder => sub {
 has 'port'           => is => 'lazy', isa => NonZeroPositiveInt,
    builder           => sub { $_[ 0 ]->_env->{ 'SERVER_PORT' } // 80 };
 
-has 'query'          => is => 'lazy', isa => SimpleStr, builder => sub {
+has 'protocol'       => is => 'lazy', isa => NonEmptySimpleStr,
+   builder           => sub { $_[ 0 ]->_env->{ 'SERVER_PROTOCOL' } };
+
+has 'query'          => is => 'lazy', isa => Str, builder => sub {
    my $v             =  $_[ 0 ]->_env->{ 'QUERY_STRING' }; $v ? "?${v}" : NUL };
 
 has 'remote_host'    => is => 'lazy', isa => SimpleStr,
@@ -103,7 +110,7 @@ has 'upload'         => is => 'lazy', isa => Object | Undef,
    predicate         => TRUE;
 
 has 'uri'            => is => 'lazy', isa => Object, builder => sub {
-   new_uri $_[ 0 ]->_base.$_[ 0 ]->path, $_[ 0 ]->scheme };
+   new_uri $_[ 0 ]->_base.$_[ 0 ]->path.$_[ 0 ]->query, $_[ 0 ]->scheme };
 
 # Private attributes
 has '_args'    => is => 'ro',   isa => ArrayRef,
@@ -386,6 +393,10 @@ C<mount_point> configuration attribute
 
 A non zero positive integer that default to 80. The default server port
 
+=item C<protocol>
+
+A non empty simple string. The protocol used by the request e.g. C<HTTP/2.0>
+
 =item C<query>
 
 The query parameters from the current request. A simple string beginning with
